package com.flyfish.framework.backup.service;

import com.flyfish.framework.backup.config.VersionProperties;
import com.flyfish.framework.backup.domain.Version;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

/**
 * 版本服务
 *
 * @author wangyu
 */
//@Service
public class VersionService {

    private final VersionProperties versionProperties;

    private final WebClient webClient;

    public VersionService(VersionProperties versionProperties) {
        this.versionProperties = versionProperties;
        this.webClient = WebClient.create(versionProperties.getServer());
    }

    /**
     * 拉取最新版本
     *
     * @return 结果
     */
    public Mono<Version> fetch() {
        return webClient.get().uri(versionProperties.getVersionUri())
                .exchangeToMono(clientResponse -> clientResponse.bodyToMono(Version.class));
    }

    /**
     * 检查新版
     *
     * @return 结果
     */
    public Mono<Boolean> checkUpdate() {
        return fetch().map(version -> !version.getVersion().equals(versionProperties.getVersion()));
    }
}
