package com.flyfish.framework.domain.authorized;

import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.query.QueryDefinition;
import com.flyfish.framework.query.Queries;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;

/**
 * 带鉴权的查询实体，主要以部门隔绝
 *
 * @param <T>
 */
@Getter
@Setter
public class AuthorizedQo<T extends AuthorizedDomain> extends AbstractAuthorizedQo<T> {

    @Override
    public QueryDefinition queryBuilder() {
        // 超级管理员拥有查看所有草稿的权限
        if (user.getType() == UserType.SUPER_ADMIN) {
            return super.queryBuilder().mutate().and(this.withPublished());
        }
        // 查询草稿，只查询自己的
        if (BooleanUtils.isFalse(published)) {
            return super.queryBuilder()
                    .mutate()
                    .and(this.withPublished())
                    .and(Queries.where(T::getCreatorId).eq(user.getId()));
        }
        // 普通查询，根据权限配置查询
        return super.queryBuilder()
                .mutate()
                .and(this.withPublished())
                .and(Queries.where(T::getAuthorizeId).in(authorizeIds())
                        .or(Queries.where(T::getCreatorId).eq(user.getId())
                                .and(T::getAuthorizeId).in(((AuthorizedUserDetails) user).getVisibleDeparts()))
                );
    }

}
