package com.flyfish.framework.domain.base;

import com.flyfish.framework.query.QueryDefinition;
import com.flyfish.framework.query.Queries;
import lombok.Getter;
import lombok.Setter;
import org.springframework.data.domain.Sort;
import org.springframework.util.StringUtils;

import java.util.Collection;
import java.util.List;

/**
 * 基于名字模糊匹配的查询实体
 */
@Getter
@Setter
public class NameLikeQo<T extends Domain> extends BaseQo<T> {

    protected String name;

    protected String code;

    protected List<String> createTimeRange;

    protected List<String> modifyTimeRange;

    protected String creatorId;

    protected String modifierId;

    protected Boolean enable;

    protected Collection<String> ids;

    protected String excludeId;

    /**
     * 排序参数
     */
    private String sort;

    /**
     * 去重字段
     */
    private String distinct;

    @Override
    public QueryDefinition queryBuilder() {
        return Queries.where("name").like(name)
                .and("enable").eq(enable)
                .and("code").eq(code)
                .and("creatorId").eq(creatorId)
                .and("modifierId").eq(modifierId)
                .and("id").in(ids)
                .and("id").ne(excludeId)
                .and("createTime").between(createTimeRange)
                .and("modifyTime").between(modifyTimeRange);
    }


    @Override
    public Sort sorts() {
        if (StringUtils.hasText(sort)) {
            return Sort.by(Sort.Order.desc(sort));
        }
        return super.sorts();
    }
}
