package com.flyfish.framework.domain.tree;

import com.flyfish.framework.domain.base.NameLikeQo;
import com.flyfish.framework.query.QueryDefinition;
import com.flyfish.framework.query.LambdaQueryChain;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;

import java.util.List;

/**
 * 属性菜单的qo
 */
@Getter
@Setter
public class TreeQo<T extends TreeDomain<T>> extends NameLikeQo<T> {

    private Integer depth;

    private String parentId;

    private List<String> parentIds;

    private Boolean recursive;

    @Override
    public QueryDefinition queryBuilder() {
        LambdaQueryChain<T> chain = super.queryBuilder().mutate().and(T::getDepth).eq(depth);
        if (BooleanUtils.isTrue(recursive)) {
            chain.and(T::getParentIds).has(parentId).and(T::getParentIds).in(parentId);
        } else {
            chain.and(T::getParentId).eq(parentId).and(T::getParentId).in(parentIds);
        }
        return chain;
    }
}
