package com.flyfish.framework.query;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.query.support.DomainFunction;

/**
 * 查询修改逻辑
 *
 * @author wangyu
 */
public interface QueryMutation {

    /**
     * 以lambda的形式修改
     * 注意，如果之前用过lambda的形式，此处应该校验类型
     *
     * @param getter 方法引用
     * @param <T>    实体泛型
     * @return 结果
     */
    <T extends Domain> QueryCondition<LambdaQueryChain<T>> and(DomainFunction<T> getter);

    /**
     * 以column的形式修改
     *
     * @param column 列名
     * @return 结果
     */
    QueryCondition<NamedQueryChain> and(String column);

    /**
     * 直接以and拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    <T extends Domain> LambdaQueryChain<T> and(LambdaQueryChain<T> chain);

    /**
     * 直接以and拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    NamedQueryChain and(NamedQueryChain chain);

    /**
     * 以lambda的形式修改
     * 注意，如果之前用过lambda的形式，此处应该校验类型
     *
     * @param getter 方法引用
     * @param <T>    实体泛型
     * @return 结果
     */
    <T extends Domain> QueryCondition<LambdaQueryChain<T>> or(DomainFunction<T> getter);

    /**
     * 以column的形式修改
     *
     * @param column 列名
     * @return 结果
     */
    QueryCondition<NamedQueryChain> or(String column);

    /**
     * 直接以or拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    <T extends Domain> LambdaQueryChain<T> or(LambdaQueryChain<T> chain);

    /**
     * 直接以or拼接其他的所有条件
     *
     * @param chain 传入的查询
     * @return 结果
     */
    NamedQueryChain or(NamedQueryChain chain);
}
