package com.flyfish.framework.repository;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.repository.base.DomainRepository;
import org.springframework.data.domain.Sort;
import org.springframework.data.repository.CrudRepository;
import org.springframework.data.repository.NoRepositoryBean;
import org.springframework.data.repository.PagingAndSortingRepository;

import java.util.List;

/**
 * 默认的仓库
 *
 * @author wangyu
 */
@NoRepositoryBean
public interface DefaultRepository<T extends Domain> extends CrudRepository<T, String>, PagingAndSortingRepository<T, String>,
        QueryModelExecutor<T>,
        DomainRepository<T> {

    /*
     * (non-Javadoc)
     * @see org.springframework.data.repository.CrudRepository#saveAll(java.lang.Iterable)
     */
    @Override
    <S extends T> List<S> saveAll(Iterable<S> entities);

    /*
     * (non-Javadoc)
     * @see org.springframework.data.repository.CrudRepository#findAll()
     */
    @Override
    List<T> findAll();

    /**
     * 插入新对象
     *
     * @param entity 实体
     * @param <S>    泛型
     * @return 插入的实体，带id
     */
    <S extends T> S insert(S entity);

    /**
     * 批量插入对象
     *
     * @param entities 对象列表
     * @param <S>      泛型
     * @return 结果列表
     */
    <S extends T> List<S> insert(Iterable<S> entities);
}
