package com.flyfish.framework.annotations.query;

import com.flyfish.framework.enums.NamedEnum;
import lombok.AllArgsConstructor;
import lombok.Getter;
import org.springframework.core.annotation.AliasFor;

import java.lang.annotation.*;

/**
 * 查询字段注解，声明映射，查询逻辑
 *
 * @author wangyu
 */
@Target(ElementType.FIELD)
@Retention(RetentionPolicy.RUNTIME)
@Documented
public @interface QueryField {

    /**
     * @return 绑定的持久层字段
     */
    @AliasFor("name")
    String value() default "";

    /**
     * @return 绑定的持久层字段
     */
    String name() default "";

    /**
     * @return 表达式类型
     */
    Type type() default Type.EQ;

    /**
     * 拼接在某个字段前
     *
     * @return
     */
    String before() default "";

    /**
     * @return 和前面条件表达式的连接方式
     */
    Logistic[] start() default Logistic.AND;

    /**
     * @return 和后面条件表达式的连接方式
     */
    Logistic[] end() default Logistic.NULL;

    /**
     * @return 当前类内排序值
     */
    int order() default 0;

    /**
     * 查询类型
     */
    @AllArgsConstructor
    @Getter
    enum Type implements NamedEnum {

        EQ("等于字段值"),
        NE("不等于字段值"),
        GT("大于字段值"),
        GTE("大于等于字段值"),
        LT("小于字段值"),
        LTE("小于等于字段值"),
        LIKE("模糊匹配字段值"),
        LIKE_LEFT("匹配左半部分字段值"),
        LIKE_RIGHT("匹配有半部分字段值"),
        IN("在字段值列表内"),
        NIN("不再字段值列表内"),
        NOT_NULL("不为空"),
        IS_NULL("为空"),
        BETWEEN("介于列表下标0和1的值之间"),
        DATE_GTE("日期大于字段值"),
        DATE_LTE("日期小于字段值");

        private final String name;
    }

    /**
     * 查询逻辑
     */
    @AllArgsConstructor
    @Getter
    enum Logistic implements NamedEnum {

        NULL("无"),
        AND("与"),
        OR("或"),
        BRACKET_START("左括号"),
        BRACKET_END("右括号");

        private final String name;
    }
}
