package com.flyfish.framework.r2dbc.operations;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.query.Query;
import com.flyfish.framework.repository.ReactiveEntityOperations;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.r2dbc.core.R2dbcEntityOperations;
import org.springframework.data.relational.repository.query.RelationalEntityInformation;
import org.springframework.data.repository.core.EntityInformation;
import org.springframework.data.util.CastUtils;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RequiredArgsConstructor
public class R2dbcReactiveEntityOperations implements ReactiveEntityOperations {

    private final R2dbcEntityOperations r2dbcEntityOperations;

    /**
     * 查询，通过查询条件和类进行精确查询
     *
     * @param query             查询
     * @param entityInformation 实体信息
     * @return 查询结果
     */
    @Override
    public <T extends Domain, E extends EntityInformation<T, String>> Mono<T> find(Query query, E entityInformation) {
        return r2dbcEntityOperations.selectOne(toNative(query, entityInformation), cast(entityInformation).getJavaType());
    }

    /**
     * 查询全部符合条件的实体
     *
     * @param query             查询
     * @param entityInformation 实体信息
     * @return 结果
     */
    @Override
    public <T extends Domain, E extends EntityInformation<T, String>> Flux<T> findAll(Query query, E entityInformation) {
        return r2dbcEntityOperations.select(toNative(query, entityInformation), cast(entityInformation).getJavaType());
    }

    private <T extends Domain> RelationalEntityInformation<T, String> cast(EntityInformation<T, String> entityInformation) {
        return CastUtils.cast(entityInformation);
    }

    /**
     * 保存实体
     *
     * @param bean 实体
     * @return 结果
     */
    @Override
    public <T extends Domain> Mono<T> save(T bean) {
        if (StringUtils.isNotBlank(bean.getId())) {
            return r2dbcEntityOperations.update(bean);
        }
        return r2dbcEntityOperations.insert(bean);
    }
}
