package com.flyfish.framework.r2dbc.query.adaptor;

import com.flyfish.framework.query.Queries;
import com.flyfish.framework.query.spi.adaptor.CriteriaAdaptor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.relational.core.query.Criteria;

import java.util.Collection;
import java.util.List;

/**
 * r2dbc查询适配器
 *
 * @author wangyu
 */
public class R2dbcCriteriaAdaptor implements CriteriaAdaptor<Criteria> {

    @Override
    public Criteria empty() {
        return Criteria.empty();
    }

    @Override
    public Criteria and(Criteria first, Criteria other) {
        return first.and(other);
    }

    @Override
    public Criteria or(Criteria first, Criteria other) {
        return first.or(other);
    }

    @Override
    public Criteria eq(String column, Object value) {
        return Criteria.where(column).is(value);
    }

    @Override
    public Criteria hasId(String column, Object value) {
        throw new UnsupportedOperationException("暂时不支持嵌套操作");
    }

    @Override
    public Criteria has(String column, Object value) {
        throw new UnsupportedOperationException("暂时不支持嵌套操作");
    }

    @Override
    public Criteria ne(String column, Object value) {
        return Criteria.where(column).not(value);
    }

    @Override
    public Criteria between(String column, List<?> items) {
        return Criteria.where(column).between(items.get(0), items.get(1));
    }

    @Override
    public Criteria like(String column, String keyword, Queries.Direction direction) {
        if (direction == Queries.Direction.LEFT) {
            keyword = keyword + "%";
        } else if (direction == Queries.Direction.RIGHT) {
            keyword = "%" + keyword;
        } else {
            keyword = StringUtils.wrap(keyword, "%");
        }
        return Criteria.where(column).like(keyword);
    }

    @Override
    public Criteria isNull(String column) {
        return Criteria.where(column).isNull();
    }

    @Override
    public Criteria lt(String column, Object value) {
        return Criteria.where(column).lessThan(value);
    }

    @Override
    public Criteria lte(String column, Object value) {
        return Criteria.where(column).lessThanOrEquals(value);
    }

    @Override
    public Criteria gt(String column, Object value) {
        return Criteria.where(column).greaterThan(value);
    }

    @Override
    public Criteria gte(String column, Object value) {
        return Criteria.where(column).greaterThanOrEquals(value);
    }

    @Override
    public Criteria in(String column, Collection<?> values) {
        return Criteria.where(column).in(values);
    }

    @Override
    public Criteria in(String column, Object... values) {
        return Criteria.where(column).in(values);
    }

    @Override
    public Criteria nin(String column, Object... values) {
        return Criteria.where(column).notIn(values);
    }

    @Override
    public Criteria nin(String column, Collection<?> values) {
        return Criteria.where(column).notIn(values);
    }
}
