package com.flyfish.framework.file.utils;

import com.flyfish.framework.context.DateContext;
import com.flyfish.framework.utils.Assert;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.FileReader;
import java.io.LineNumberReader;
import java.util.Date;

public abstract class SystemUtils {

    /**
     * 判断是否是windows系统
     *
     * @return 结果
     */
    public static boolean isWindows() {
        String os = System.getProperty("os.name");
        return os.toLowerCase().startsWith("win");
    }

    public static String generateDatePath() {
        return DateContext.formatDate(new Date(), "yyyy/MM/dd");
    }

    public static String generateDatePath(Date date) {
        return DateContext.formatDate(date, "yyyy/MM/dd");
    }

    /**
     * 根据当前位置生成按照指定步长的区间
     *
     * @param pos  位置
     * @param step 步长
     * @return 结果
     */
    public static String range(long pos, int step) {
        // 求余
        long left = pos % step;
        // 求整数商
        long times = pos / step;
        // 得出区间
        long min = times * step;
        long max = (times + 1) * step;
        // 返回
        return min + "-" + max;
    }

    /**
     * 根据基本路径生成时间戳（年/月/日）的目录接口，并创建文件
     *
     * @param fullPath 基本路径
     * @param fileName 文件名
     * @return 结果
     */
    public static File createFile(String fullPath, String fileName) {
        testPath(fullPath);
        return new File(fullPath + fileName);
    }

    /**
     * 测试路径，并创建
     *
     * @param fullPath 全路径
     */
    public static void testPath(String fullPath) {
        if (fullPath.contains(".")) {
            fullPath = getPath(fullPath);
        }
        File directory = new File(fullPath);
        if (!directory.exists()) {
            if (!directory.mkdirs()) {
                Assert.isTrue(directory.exists(), "目录创建失败！");
            }
        }
    }


    public static String getPath(String fullPath) {
        return StringUtils.substringBeforeLast(fullPath, "/");
    }

    public static long getLineNumber(String fullPath) throws Exception {
        try (FileReader in = new FileReader(fullPath);
             LineNumberReader reader = new LineNumberReader(in)) {
            long skipped = reader.skip(Long.MAX_VALUE);
            if (skipped == 0) {
                return 0;
            }
            return reader.getLineNumber();
        }
    }
}
