package com.flyfish.framework.user.config.captcha;

import com.dingxianginc.ctu.client.CaptchaClient;
import com.dingxianginc.ctu.client.model.CaptchaResponse;
import com.flyfish.framework.user.config.properties.SecurityProperties;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.security.authentication.BadCredentialsException;

/**
 * 顶象验证码验证器
 *
 * @author wangyu
 */
@Slf4j
public class DxCaptchaValidator {

    private final CaptchaClient captchaClient;

    public DxCaptchaValidator(SecurityProperties securityProperties) {
        SecurityProperties.CaptchaProperties captchaProperties = securityProperties.getCaptcha();
        this.captchaClient = new CaptchaClient(captchaProperties.getAppId(), captchaProperties.getAppSecret());
        captchaClient.setCaptchaUrl("https://" + captchaProperties.getServer() + "/api/tokenVerify");
    }

    /**
     * 执行验证码验证
     *
     * @param token 验证码
     * @return 结果
     */
    public void verify(String token) {
        //指定服务器地址，saas可在控制台，应用管理页面最上方获取
        try {
            CaptchaResponse response = captchaClient.verifyToken(token);
            log.info(response.getCaptchaStatus());
            if (BooleanUtils.isNotTrue(response.getResult())) {
                throw new BadCredentialsException("验证码不正确！");
            }
        } catch (Exception e) {
            log.error("验证码验证失败！" + e.getMessage());
            throw new BadCredentialsException("验证码验证异常！");
        }
    }
}
