package com.flyfish.framework.user.controller;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.user.config.RSAKeys;
import com.flyfish.framework.user.config.properties.SecurityProperties;
import com.flyfish.framework.configuration.jwt.TokenProvider;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.ReactiveSecurityContextHolder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;

/**
 * 登录controller
 *
 * @author wangyu
 */
@RestController
@RequestMapping("/login")
public class LoginController {

    @Resource
    private SecurityProperties securityProperties;
    @Resource
    private TokenProvider tokenProvider;

    /**
     * 获取公钥，如果有的话
     *
     * @return 结果
     */
    @GetMapping("")
    public Result<String> getPublicKey() {
        if (securityProperties.isRsa()) {
            return Result.ok(RSAKeys.PUBLIC_KEY);
        }
        return Result.error("尚未配置加密密钥！");
    }

    /**
     * 刷新token，返回新token
     * 这里不销毁原来的token，等他自动过期
     * @return 结果
     */
    @GetMapping("refresh")
    public Mono<Void> refreshToken(ServerWebExchange exchange) {
        return ReactiveSecurityContextHolder.getContext()
                .doOnNext(securityContext -> {
                    Authentication authentication = securityContext.getAuthentication();
                    tokenProvider.addToken(exchange, authentication);
                })
                .then();
    }
}
