package com.flyfish.framework.user.service.impl;

import com.flyfish.framework.user.domain.AdminUserDetails;
import com.flyfish.framework.user.domain.DepartmentQo;
import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.enums.UserType;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import reactor.core.publisher.Mono;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.function.Function;

import static org.apache.commons.lang3.ObjectUtils.defaultIfNull;

class DepartmentServiceHelper {

    /**
     * 转换qo，经过可见部门转换，更加私密
     *
     * @param query 查询
     * @return 结果
     */
    static Mono<Qo<Department>> convertQo(Qo<Department> query, Function<Set<String>, Mono<Integer>> minDeptIds) {
        if (query instanceof DepartmentQo) {
            DepartmentQo qo = (DepartmentQo) query;
            // 如果是非管理员
            if (null != qo.getUser() && qo.getUser().getType() != UserType.SUPER_ADMIN) {
                // 占位，包装查询实体
                Mono<Qo<Department>> mono = Mono.just(qo);
                // 使用列表内的部门作为条件
                AdminUserDetails userDetails = (AdminUserDetails) query.getUser();
                // 是否设置过
                qo.setIds(userDetails.getVisibleDeparts());
                // 查询根节点下的节点
                if (Department.ROOT.equals(qo.getParentId())) {
                    qo.setParentId(null);
                    // 不递归，指定深度，保证单层
                    if (BooleanUtils.isNotTrue(qo.getRecursive())) {
                        // 指定深度，从可见的列表中获取
                        mono = minDeptIds.apply(userDetails.getVisibleDeparts())
                                .map(depth -> {
                                    qo.setDepth(depth);
                                    return qo;
                                });
                    }
                }
                // 如果存在额外的id，追加记录
                if (StringUtils.isNotBlank(qo.getAdditional())) {
                    Collection<String> ids = defaultIfNull(qo.getIds(), new HashSet<>());
                    ids.add(qo.getAdditional());
                    qo.setIds(ids);
                }
                // 如果设置过ids，但是qo中ids为空，返回无效查询
                if (CollectionUtils.isEmpty(qo.getIds())) {
                    qo.setIds(Collections.singleton("null"));
                }
                return mono;
            }
        }
        return Mono.just(query);
    }
}
