package com.flyfish.framework.beans.meta;

import com.flyfish.framework.controller.SafeController;
import com.flyfish.framework.controller.reactive.ReactiveBaseController;
import com.flyfish.framework.domain.base.DomainService;
import com.flyfish.framework.domain.base.NameLikeQo;
import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.domain.base.Vo;
import com.flyfish.framework.repository.DefaultReactiveRepository;
import com.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import org.springframework.core.annotation.AliasFor;
import org.springframework.data.mongodb.core.mapping.Document;
import org.springframework.data.relational.core.mapping.Table;
import org.springframework.data.repository.Repository;

import java.lang.annotation.*;

/**
 * RestBean扫描注解
 *
 * @author wangyu
 */
@Target({ElementType.TYPE})
@Retention(RetentionPolicy.RUNTIME)
@Documented
@Document
@Table
public @interface RestBean {

    /**
     * 生成的REST资源名称
     *
     * @return 资源名称
     */
    @AliasFor(value = "collection", annotation = Document.class)
    String value() default "";

    /**
     * 表名
     *
     * @return 物理表名
     */
    @AliasFor(value = "value", annotation = Table.class)
    String table() default "";

    /**
     * 名称，可用于占位
     *
     * @return 结果
     */
    String name() default "";

    /**
     * 排除的属性
     *
     * @return 结果
     */
    String[] exclude() default {};

    /**
     * 元数据，可以自由地指定一些值
     *
     * @return 结果
     */
    String[] meta() default {};

    /**
     * 必须指定qo
     *
     * @return 结果
     */
    Class<? extends Qo> queryClass() default NameLikeQo.class;

    /**
     * 指定列表视图
     *
     * @return 结果
     */
    Class<? extends Vo> listViewClass() default Vo.class;

    /**
     * 指定详情视图
     *
     * @return 结果
     */
    Class<? extends Vo> viewClass() default Vo.class;

    /**
     * 仓库的超类，默认是default
     *
     * @return 结果
     */
    Class<? extends Repository> repoClass() default DefaultReactiveRepository.class;

    /**
     * 服务类，动态生成服务
     *
     * @return 结果
     */
    Class<? extends DomainService> serviceClass() default BaseReactiveServiceImpl.class;

    /**
     * controller的类，支持自定义
     *
     * @return 结果
     */
    Class<? extends SafeController> controllerClass() default ReactiveBaseController.class;
}
