package com.flyfish.framework.beans.meta.parser.chain.impl;

import com.flyfish.framework.beans.meta.parser.BeanPropertyAnnotations;
import com.flyfish.framework.beans.meta.parser.chain.BeanPropertyAnnotationChain;
import org.springframework.core.annotation.MergedAnnotation;
import org.springframework.data.util.CastUtils;

import java.lang.annotation.Annotation;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;

/**
 * 简单的注解链
 *
 * @param <A> 注解类型
 * @param <T> 泛型
 */
public class SimpleAnnotationChain<A extends Annotation, T> extends BasicAnnotationChainSupport
        implements BeanPropertyAnnotationChain<A, T> {

    // 内置值
    private T value;

    public SimpleAnnotationChain(BeanPropertyAnnotations parent, T value) {
        super(parent);
        this.value = value;
    }

    /**
     * 映射转换
     *
     * @param mapper 映射器
     * @return 结果
     */
    @Override
    public <R> BeanPropertyAnnotationChain<A, R> map(Function<T, R> mapper) {
        R value = mapper.apply(this.value);
        if (null == value) {
            return parent.empty();
        }
        this.value = CastUtils.cast(value);
        return CastUtils.cast(this);
    }

    /**
     * 过滤，不满足的将直接empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> filter(Predicate<T> filter) {
        // 测试不通过，直接返回空，释放内存
        if (!filter.test(this.value)) {
            return parent.empty();
        }
        return this;
    }

    /**
     * 过滤，不满足的将直接empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> filter(Supplier<Boolean> filter) {
        if (!filter.get()) {
            return parent.empty();
        }
        return this;
    }

    /**
     * 如果存在，则做
     *
     * @param consumer 消费者
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> then(Consumer<T> consumer) {
        parent.last(true);
        consumer.accept(value);
        return this;
    }

    /**
     * 存在时执行逻辑，不需要结果
     *
     * @param handler 处理器
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> exists(Runnable handler) {
        parent.last(true);
        handler.run();
        return this;
    }

    /**
     * 不存在做的事情
     *
     * @param fallback 补偿逻辑
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> empty(Runnable fallback) {
        return this;
    }

    /**
     * 直接获取结果
     *
     * @return 结果
     */
    @Override
    @SuppressWarnings("unchecked")
    public A synthesize() {
        parent.last(false);
        return value instanceof MergedAnnotation ? ((MergedAnnotation<A>) value).synthesize() : null;
    }
}
