package com.flyfish.framework.beans.repository;

import com.flyfish.framework.repository.DefaultReactiveRepository;
import com.flyfish.framework.repository.DefaultRepository;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.BeanNameGenerator;
import org.springframework.core.env.Environment;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.type.AnnotationMetadata;
import org.springframework.data.mongodb.repository.config.EnableMongoRepositories;
import org.springframework.data.mongodb.repository.config.EnableReactiveMongoRepositories;
import org.springframework.data.mongodb.repository.config.MongoRepositoryConfigurationExtension;
import org.springframework.data.mongodb.repository.config.ReactiveMongoRepositoryConfigurationExtension;
import org.springframework.data.r2dbc.repository.config.EnableR2dbcRepositories;
import org.springframework.data.r2dbc.repository.config.R2dbcRepositoryConfigurationExtension;

import java.util.ArrayList;
import java.util.List;

/**
 * 注册器组合对象
 *
 * @author wangyu
 */
class RepositoryRegistrarComposite implements RepositoryRegistrar {

    private final List<RepositoryRegistrar> registrars;

    private final String ENABLE_MONGO_REPO_ANNO = "org.springframework.data.mongodb.repository.config.EnableMongoRepositories";

    private final String ENABLE_REACTIVE_MONGO_REPO_ANNO = "org.springframework.data.mongodb.repository.config.EnableReactiveMongoRepositories";

    private final String ENABLE_R2DBC_REPO_ANNO = "org.springframework.data.r2dbc.repository.config.EnableR2dbcRepositories";

    RepositoryRegistrarComposite(AnnotationMetadata metadata) {
        List<RepositoryRegistrar> list = new ArrayList<>();
        if (metadata.hasMetaAnnotation(ENABLE_MONGO_REPO_ANNO)) {
            list.add(new SimpleRepositoryRegistrar(EnableMongoRepositories.class, new MongoRepositoryConfigurationExtension(),
                    DefaultRepository.class));
        }
        if (metadata.hasMetaAnnotation(ENABLE_REACTIVE_MONGO_REPO_ANNO)) {
            list.add(new SimpleRepositoryRegistrar(EnableReactiveMongoRepositories.class, new ReactiveMongoRepositoryConfigurationExtension(),
                    DefaultReactiveRepository.class));
        }
        if (metadata.hasMetaAnnotation(ENABLE_R2DBC_REPO_ANNO)) {
            list.add(new SimpleRepositoryRegistrar(EnableR2dbcRepositories.class, new R2dbcRepositoryConfigurationExtension(),
                    DefaultReactiveRepository.class));
        }
        this.registrars = list;
    }

    @Override
    public RepositoryRegistrar init(AnnotationMetadata metadata, ResourceLoader resourceLoader, Environment environment,
                                    BeanDefinitionRegistry registry, BeanNameGenerator generator) {
        registrars.forEach(registrar -> registrar.init(metadata, resourceLoader, environment, registry, generator));
        return this;
    }

    @Override
    public void register(Class<?> clazz) {
        if (null == clazz) return;
        registrars.forEach(registrar -> registrar.register(clazz));
    }
}
