package com.flyfish.framework.beans.resolver;

import com.flyfish.framework.compiler.DynamicJavaCompiler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.context.annotation.Lazy;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * 动态Bean生成解析器
 *
 * @author wangyu
 */
@Slf4j
@Lazy(false)
public class DynamicRestBeanResolver implements DisposableBean {

    private static final Map<String, Class<?>> classes = new HashMap<>();

    public static void register(String name, Class<?> clazz) {
        classes.put(name, clazz);
    }

    public Collection<Class<?>> getClasses() {
        return classes.values();
    }

    public Class<?> getClass(String name) {
        return classes.get(name);
    }

    @Override
    public void destroy() throws Exception {
        log.info("正在销毁rest自动配置器");
        DynamicJavaCompiler.delegate().close();
    }
}
