package com.flyfish.framework.service;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.transform.DataBufferTransformer;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.web.server.WebFilterExchange;
import reactor.core.publisher.Mono;

/**
 * 登录审计
 *
 * @author wangyu
 */
public interface AuthenticationAuditor extends DataBufferTransformer<Result<?>> {

    /**
     * 登录成功的回调
     *
     * @param userDetails 用户信息
     */
    Mono<Void> success(UserDetails userDetails);

    /**
     * 登录失败的回调
     *
     * @param webFilterExchange 请求信息
     * @param exception         异常
     */
    Mono<Void> error(WebFilterExchange webFilterExchange, AuthenticationException exception);

    /**
     * 退出登录的回调
     *
     * @param userDetails 用户详情
     */
    Mono<Void> logout(UserDetails userDetails);
}
